@@ -440,6 +440,11 @@ def data_from_dataset(dataset, X_indexing=None, y_indexing=None):
       If not None, use this function for indexing into the y data. If
       None, try to automatically determine how to index data.
 
+    Raises
+    ------
+    AttributeError
+      If X and y could not be accessed from the dataset.
+
     """
     X, y = _none, _none
 
@@ -450,6 +455,9 @@ def data_from_dataset(dataset, X_indexing=None, y_indexing=None):
         y = multi_indexing(y, dataset.indices, indexing=y_indexing)
     elif hasattr(dataset, 'X') and hasattr(dataset, 'y'):
         X, y = dataset.X, dataset.y
+    elif isinstance(dataset, torch.utils.data.dataset.TensorDataset):
+        if len(items := dataset.tensors) == 2:
+            X, y = items
 
     if (X is _none) or (y is _none):
         raise AttributeError("Could not access X and y from dataset.")
@@ -634,6 +642,7 @@ def _sigmoid_then_2d(x):
     # return x.T
 
 
+# pylint: disable=protected-access
 def _infer_predict_nonlinearity(net):
     """Infers the correct nonlinearity to apply for this net
 
@@ -646,7 +655,11 @@ def _infer_predict_nonlinearity(net):
     # based on the criterion, not the class of the net. We still pass
     # the whole net as input in case we want to modify this at a
     # future point in time.
-    criterion = net.criterion_
+    if len(net._criteria) != 1:
+        # don't know which criterion to consider, don't try to guess
+        return _identity
+
+    criterion = getattr(net, net._criteria[0] + '_')
 
     if isinstance(criterion, CrossEntropyLoss):
         return partial(torch.softmax, dim=-1)
